#version 330 core

in vec2 UV;
in vec3 PosWorld;
in vec3 NormalView;

layout(location = 0) out vec4 color;

struct Light 
{
	vec3 	lightPosWorld;
	float 	lightPower;
	float	constant;
	float	linear;
	float	quadratic;
};
struct Material {
    vec3 	ambient;
    vec3 	diffuse;
    vec3 	specular;
    float 	shininess;
}; 

#define 			MAXLIGHTS 64
uniform sampler2D	textureSampler;
uniform int			countLights;
uniform Light		light[MAXLIGHTS];
uniform Material 	material;
uniform float 		alpha;
uniform bool		isCelShading;
uniform float		ambientFraction 	= 0.1;
uniform float		specularFraction 	= 0.05;
uniform vec3 		lightColor			= vec3(0.960, 0.537, 0.537);
const float 		levels 				= 4.0;

vec3 calculatePointLight(Light light, vec3 materialDiffuseColor);

void main()
{
	vec3	materialDiffuseColor	= texture(textureSampler, UV).rgb;
	vec3	sumColorPointLight		= vec3(0,0,0);

	for(int i = 0; i < countLights; i++)
	{
		sumColorPointLight += calculatePointLight(light[i], materialDiffuseColor);
	}	
	color = vec4(sumColorPointLight.xyz, alpha);
}

vec3 calculatePointLight(Light light, vec3 materialDiffuseColor)
{
	//calculate angle of normal and lightdir
	vec3	lightDir		= light.lightPosWorld - PosWorld;
	
	vec3	n				= normalize(NormalView);
	vec3	l				= normalize(lightDir);
	vec3	e				= normalize(-light.lightPosWorld);
	vec3	r				= reflect(-l, n);
	
	float	cosTheta		= max(dot(n, l), 0);
	float	spec			= pow(max(dot(e, r), 0), 32);
	float	distance		= length(lightDir);
	vec3 	attenuation 	= lightColor * light.lightPower / (light.constant + light.linear * distance + light.quadratic * (distance * distance));
	
	if(isCelShading) {
		float level = floor(cosTheta * levels);
		cosTheta = level / levels;
	}
	
	vec3	ambientPart		= ambientFraction * materialDiffuseColor;
	vec3	diffusePart		= materialDiffuseColor * cosTheta * attenuation;
	vec3	specularPart	= specularFraction * material.specular * spec * attenuation;

	return ambientPart + diffusePart + specularPart; 
}