#version 330 core

in vec4 clipSpace;
in vec2 textureCoords;

out vec4 color;

uniform sampler2D reflectionTexture;
uniform sampler2D refractionTexture;
uniform sampler2D dudvMap;

uniform float moveFactor;

const float waveStrength = 0.02;

void main() {

	vec2 distortion1 		= (texture(dudvMap, vec2(textureCoords.x, textureCoords.y)).rg * 2.0 - 1.0) * waveStrength;

	vec2 ndc 				= (clipSpace.xy / clipSpace.w) / 2.0 + 0.5;
	
	vec2 refractTexCoords 	= clamp(ndc + distortion1, 0.001, 0.999);
	vec2 reflectTex 		= vec2(ndc.x, -ndc.y) + distortion1;
	reflectTex.x = clamp(reflectTex.x, 0.001, 0.999);
	reflectTex.y = clamp(reflectTex.y, -0.999, -0.001);
	
	vec4 reflectColor 		= texture(reflectionTexture, reflectTex);
	vec4 refractionColor 	= texture(refractionTexture, refractTexCoords);

	
	
    color = mix(reflectColor, refractionColor, 0.5);
}