#version 330

layout(points) in;
layout(points) out;
layout(max_vertices = 30) out;

in float	type[];
in vec3 	position[];
in vec3 	direction[];
in float 	age[];

out float 	typeVarying;
out vec3 	positionVarying;
out vec3 	directionVarying;
out float 	ageVarying;

uniform bool isSpawn = false;
uniform float deltaTime;
uniform float launchTime;
uniform float fireTime;
uniform float spreadTime;
uniform int randomTextureSize;
uniform float directionParticles;
uniform sampler1D randomTexture;  
uniform float FRONT = 0.0f;
uniform float BACK 	= 1.0f;
uniform float LEFT 	= 2.0f;
uniform float RIGHT = 3.0f;
uniform float UP 	= 4.0f;
uniform float DOWN 	= 5.0f;

#define PARTICLE_TYPE_LAUNCHER 0.0f
#define PARTICLE_TYPE_SHELL 1.0f
#define PARTICLE_TYPE_SECONDARY_SHELL 2.0f

vec3 setDirection(vec3 currentDirection, float selectedDir)
{
	if(selectedDir == FRONT){
		currentDirection.x = 1.0f;
	}
	else if(selectedDir == BACK){
		currentDirection.x = -1.0f;
	}
	else if(selectedDir == LEFT){
		currentDirection.z = -1.0f;
	}
	else if(selectedDir == RIGHT){
		currentDirection.z = 1.0f;
	}
	else if(selectedDir == UP){
		currentDirection.y = 1.0f;
	}
	else if(selectedDir == DOWN){
		currentDirection.y = -1.0f;
	}
	return currentDirection;
}

vec3 getRandomDir(float texCoord)                                                   
{                                                                                   
    vec3 dir = texture(randomTexture, texCoord).xyz;                              
    dir -= vec3(0.5, 0.5, 0.5);
	
	
    return normalize(dir) * 0.15f;                                                                    
}                                      

void main()
{
    float currentAge = age[0] + deltaTime;
	if (type[0] == PARTICLE_TYPE_LAUNCHER) {
		if (currentAge >= launchTime) 
		{
			for (int i = 0 ; i < 5; i++) {
				if(isSpawn)
				{
					typeVarying 		= PARTICLE_TYPE_SHELL;
					positionVarying 	= position[0];
					directionVarying 	= setDirection(direction[0] / 2, directionParticles);
					ageVarying 			= 0.0;
					EmitVertex();
					EndPrimitive();
					currentAge = 0.0;
				}
			}
		}

		typeVarying 		= PARTICLE_TYPE_LAUNCHER;
		positionVarying 	= position[0] ;
		directionVarying 	= setDirection(direction[0], directionParticles);
		ageVarying 			= currentAge;
		EmitVertex();
		EndPrimitive();
    }
	else if(type[0] == PARTICLE_TYPE_SHELL) 
	{
		vec3 dir = deltaTime * direction[0] * 0.07f;
		if (currentAge < fireTime) 
		{	
			typeVarying 		= PARTICLE_TYPE_SHELL;
			positionVarying 	= position[0] + dir;
			directionVarying 	= direction[0];
			ageVarying 			= currentAge;
			EmitVertex();
			EndPrimitive();
		}
		else 
		{ 			
			typeVarying 		= PARTICLE_TYPE_SECONDARY_SHELL;                         
			positionVarying 	= position[0] + dir;                                      		               
			directionVarying 	= setDirection(getRandomDir(currentAge * randomTextureSize), directionParticles);                             
			ageVarying 			= 0.0f;                                                   
			EmitVertex();                                                  
			EndPrimitive();
		}                                                                      
    }                                                                           
	else {
		if (currentAge < spreadTime) {                              
			vec3 dir = deltaTime * direction[0];	                                  
			typeVarying 		= PARTICLE_TYPE_SECONDARY_SHELL;                              
			positionVarying 	= position[0] + dir;                                  
			directionVarying 	= direction[0];                                  
			ageVarying 			= currentAge;                                                         
			EmitVertex();                                                       
			EndPrimitive();                                                     
		}
	} 
}